!!$=head1 NAME
!!$
!!$Wilson - Wilson's factorization for helix filters
!!$
!!$=head1 SYNOPSIS
!!$
!!$Wilson < filt.H  niter=20 [n1= lag=] > minphase.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Reads a helix autocorrelation (positive side of it). 
!!$Outputs its minimum-phase factor. 
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from/to history
!!$
!!$B<integer> I<n1> - filter size
!!$
!!$B<integer array> I<lag> - comma-separated list of filter lags
!!$
!!$B<real> I<a0>=1 - zero-lag coefficient
!!$
!!$=item from par
!!$
!!$B<integer> I<niter>=20 - number of Newton's iterations
!!$
!!$B<integer> I<n1> (optional) - number of coefficients
!!$
!!$B<integer array> I<lag> (optional) - comma-separated list of filter lags
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/hlx/helderiv/>, F<gee/hlx/helicon90/>, F<gee/hlx/helocut90/>,
!!$F<gee/prc/galilee90/>
!!$
!!$=head1 MODULES
!!$
!!$F<wilson.lop>, F<helix.r90>, F<compress.r90>
!!$
!!$=head1 SEA ALSO
!!$
!!$L<Helicon>, L<Pef>, L<Autocorr>
!!$
!!$=cut
!%
program Burg
  use sep
  use helix
  use wilson
  use compress

  implicit none
  integer       :: ns, i, na, niter, maxlag
  real          :: a0, s0,small
  type (filter) :: ss, aa

  call sep_init (SOURCE)
  call from_history (ns)
  call allocatehelix (ss, ns)
  call from_history ("lag", ss%lag, (/ (i, i = 1, ns) /))
  maxlag = maxval(ss%lag)
  call from_history ("a0", s0, 1.)
  call from_par ("niter",niter,20)
  call from_par ("n1",na,maxlag)
  call allocatehelix (aa, na)
  call from_par ("lag",aa%lag,  (/ (i, i = 1, na) /))
  a0 = 1.; aa%flt = 0.
  call to_history ("a0",a0)
  call to_history ("lag",aa%lag)
  call to_history ("n1", size (aa%lag))

  call sep_read (ss%flt)
  call wilson_init (maxlag*10)
  call wilson_factor (niter, 2.*s0, ss, a0, aa, .true.)
  call wilson_close ()
  call from_param("small",small,.0000001)
  call resize (aa,small)
  call to_history ("a0",a0)
  call to_history ("lag",aa%lag)
  call to_history ("n1", size (aa%lag))
  call sep_close ()

  call sep_write (a0*aa%flt)
  call deallocatehelix (ss)
  call deallocatehelix (aa)
  call exit (0)
end program Burg












