module oc_utils_mod
  !!=======================
  !! Paul Sava    (C) 2000
  !! paul@sep.stanford.edu
  !! oclib v2.0
  !!=======================
  implicit none
  integer,parameter,private :: big_int=selected_int_kind(11)



  interface message
     module procedure print_integer
     module procedure print_real
     module procedure print_character
     module procedure print_logical
     module procedure print_info
  end interface

contains
  !----------------------------------------------------------------
  integer function factor(n,max_mem ) result(m)
    integer, intent(in) :: n(:),max_mem
    integer             ::  p,i,ireturn
    integer(kind=big_int) :: nsize1,nsize2
    logical :: found

    if(size(n)==1) then
       m=prime(product(n),max_mem)
       return
    end if

    nsize1=1;i=1;found=.false.
    do while( i <= size(n) .and. .not. found)
       nsize1=nsize1*n(i)
       if(max_mem < nsize1) then
         nsize1=nsize1/n(i) 
         found=.true.
       end if
       i=i+1
    end do
    nsize2=1;i=size(n);found=.false.
    do while( i >=1 .and. .not. found)
       nsize2=nsize2*n(i)
       if(max_mem < nsize2) then
         nsize2=nsize2/n(i) 
         found=.true.
       end if
       i=i-1
    end do
    
    m=max(nsize1,nsize2)
  end function factor
  !----------------------------------------------------------------
  integer function prime(nn, n) result(m)
    !! returns m so that N/m=integer
    integer, intent(in) :: nn,n
    integer             ::  p,i
    p=nn/n
    do m=n,1,-1
       i=p
       do while(i*m<nn)
          i=i+1
       end do
       if (i*m .eq. nn) exit
    end do
  end function prime
  !----------------------------------------------------------------
  subroutine print_integer(n,v)
    character(len=*), intent(in) :: n
    integer,          intent(in) :: v
    call ocline()
    write (0,'(" OC ",a,"=",i8)') trim(n),v
    call ocline()
  end subroutine print_integer
  subroutine print_real(n,v)
    character(len=*), intent(in) :: n
    real,             intent(in) :: v
    call ocline()
    write (0,'(" OC ",a,"=",f15.6)') trim(n),v
    call ocline()
  end subroutine print_real
  subroutine print_character(n,v)
    character(len=*), intent(in) :: n,v
    call ocline()
    write (0,'(" OC ",a,"=",a)') trim(n),trim(v)
    call ocline()
  end subroutine print_character
  subroutine print_info(n)
    character(len=*), intent(in) :: n
    call ocline()
    write (0,'(" OC ",a)') trim(n)
    call ocline()
  end subroutine print_info
  subroutine print_logical(n,v)
    character(len=*), intent(in) :: n
    logical,          intent(in) :: v
    call ocline()
    write (0,'(" OC ",a,"=",l2)') trim(n),v
    call ocline()
  end subroutine print_logical
  !----------------------------------------------------------------
  subroutine ocline()
    write (0,*) 'OC-------------------------------------------'
  end subroutine ocline
  !----------------------------------------------------------------
end module oc_utils_mod








